/*
 * Wanderer.java
 *
 * Created on June 21, 2007, 4:01 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package batch;
import java.awt.*;
import sim.engine.*;
import sim.util.*;
import ec.util.*;

/**
 *
 * @author Edward
 */
public class Macro extends Agent implements Steppable{
    //Variables and arrays used to count nearby cells in past 100 timesteps
    int age = 0;
    public int[] necRecord = new int[200];
    public int[] apopRecord = new int[200];
    double memoryNec;
    double memoryApop;
    
    //Rate constants for recurrence relations:
    //Detection of nearby cells
    double kApop1 = 0.05;
    double kApop2 = 0.5;
    double kNec1 = 0.05;
    double kNec2 = 0.5;
    
    //Levels of signalling molecules
    double kBeta1 = 0.00185;
    double kBeta2 = 0.5;
    double kAlpha1 = 0.00278;
    double kAlpha2 = 0.5;

    //Parameters controlling homeostatic production levels of signalling molecules
    double kHomBeta = 0.333;   
    double kHomAlpha = 0.0909;
    
    //Numbers of receptors
    double NApop = 100;   
    double NNec = 100;
    double NBeta = 100;
    double NAlpha = 100;
    
    //Initial number of receptors occupied
    double RApop = 0;   
    double RNec = 0;
    double RBeta = 10;
    double RAlpha = 10;
    
    //Maximum production of signalling molecules per timestep (passed in when instantiated)
    double PAlphaMax;  
    double PBetaMax;
    
    double PBeta;
    double PAlpha;
    
    double CApop;
    double CNec;
    
    double QBeta = 1;
    double QAlpha = 1;
    
    double kDeg = 0.0015;
    
    double amountBeta;
    double amountAlpha;
    
    //Identification number of agent for writing into data array
    int id;
    
    //Methods to output values to GUI interface
    public int getAge() {return age;}
    public double getRApop() {return RApop;}
    public double getRNec() {return RNec;}
    public double getRBeta() {return RBeta;}
    public double getRAlpha() {return RAlpha;}
    public int getNumApopEaten() {return numApopEaten;}
    public int getNumNecEaten() {return numNecEaten;}
    
    /** Creates a new instance of Macro agent*/
    public Macro(double v, double a, int i, double PAMax, double PBMax) {
        super(v,a);
        this.diam = 5;
        id = i;
        PAlphaMax = PAMax;
        PBetaMax = PBMax;
        this.agentCol = Color.orange;
        isaPhage = true;
        setState(agentCol, diam);
        initArray();
    }
    
    /**Sets values of array elements to zero*/
    void initArray() {
        for (int i = 0; i < necRecord.length; i++) {
            necRecord[i] = 0;
            apopRecord[i] = 0;
        }

    }
      
    /**Calculates number of occupied receptors detecting the presence of
     *apoptotic and necrotic cells based on values in apopRecord and
     necRecord arrays.*/
    void findNoOfIdentReceptors() {
        memoryApop = 0;
        memoryNec = 0;
        for (int i = 0; i < apopRecord.length; i++) {
            memoryApop += apopRecord[i];
            memoryNec += necRecord[i];
        }
        CApop = memoryApop;
        CNec = memoryNec;
        RApop += kApop1*CApop*(1 - (RApop/NApop)) - kApop2*(RApop/NApop);
        RNec += kNec1*CNec*(1 - (RNec/NNec)) - kNec2*(RNec/NNec);
        
    }
    
    /**Calculates number of occupied receptors based on concentrations
     of signalling molecules in the simulation.*/
    void findNoOfSignalReceptors(double CBeta, double CAlpha) {
        RBeta += kBeta1*CBeta*(1 - (RBeta/NBeta)) - kBeta2*(RBeta/NBeta);
        RAlpha += kAlpha1*CAlpha*(1 - (RAlpha/NAlpha)) - kAlpha2*(RAlpha/NAlpha);
    }
    
    /**Calculates level of production of signalling molecules based on number
     of occupied receptors above.*/
    void calculateSignalProduction() {
        PBeta = PBetaMax*kHomBeta*(1 + ((1 - kHomBeta)/kHomBeta)*(RApop/NApop) - (RNec/NNec));
        PAlpha = PAlphaMax*kHomAlpha*(1 + ((1 - kHomAlpha)/kHomAlpha)*(RNec/NNec) - (RApop/NApop));
    }
    
    /**Finds change in concentration based on production of signalling molecules,
     *the number binding and dissociating from receptors and a degradation constant
     kDeg.*/
    void findConcentrationIncrements(double CBeta, double CAlpha) {
        amountBeta = (PBeta + kBeta2*(RBeta/NBeta) - kBeta1*CBeta*(1 - (RBeta/NBeta)) - CBeta*kDeg)*QBeta;
        amountAlpha = (PAlpha + kAlpha2*(RAlpha/NAlpha) - kAlpha1*CAlpha*(1 - (RAlpha/NAlpha)) - CAlpha*kDeg)*QAlpha;
    }
    
    /**Methods called each timestep*/
    public void step (SimState state) {
        Engine simulation = (Engine) state;
        age++;
        age%=apopRecord.length;
        if(simulation.schedule.time()%50==0){
            simulation.writeMacroData(id, numApopEaten, numNecEaten, RApop, RNec, RAlpha, RBeta, PAlpha, PBeta);
        }
        simulation.findNewDirection(this);
        findNoOfIdentReceptors();
        findNoOfSignalReceptors(simulation.CBeta,simulation.CAlpha);
        calculateSignalProduction();
        findConcentrationIncrements(simulation.CBeta,simulation.CAlpha);
        simulation.adjustConcentration(amountBeta,amountAlpha);
        moveAgent();
        simulation.setObjectLocation(this, loc);
        simulation.collision(this);
    }
    
}
